// Copyright (C) 2023 OWO Games - All Rights Reserved
#include <OWOImport/Public/ImportObjectFactory.h>
#include "Containers/UnrealString.h"
#include "Misc/FileHelper.h"
#include "Templates/SharedPointer.h"
#include "EditorFramework/AssetImportData.h"


UImportObjectFactory::UImportObjectFactory(const FObjectInitializer& ObjectInitializer) : Super(ObjectInitializer)
{
    Formats.Add(FString(TEXT("owo;")) + NSLOCTEXT("UImportObjectFactory", "SensationFormat", "Sensation file").ToString());
    SupportedClass = UImportedSensation::StaticClass();
    bCreateNew = false;
    bEditorImport = true;
    Formats.Add(TEXT("owo; OWO Sensation"));
}

UObject* UImportObjectFactory::FactoryCreateFile(UClass* InClass, UObject* InParent, FName InName, EObjectFlags Flags, const FString& Filename, const TCHAR* Parms, FFeedbackContext* Warn, bool& bOutOperationCanceled)
{
    FString result;

    FFileHelper::LoadFileToString(result, *Filename);

    std::string code = std::string(TCHAR_TO_UTF8(*result));
    
    UImportedSensation* OWOSensation = NewObject<UImportedSensation>(InParent, UImportedSensation::StaticClass(), InName, Flags);
   
    OWOSensation->SensationsData = code.c_str();

    OWOSensation->AssetImportData->Update(Filename);

	return OWOSensation;
}

UImportedSensation* UImportObjectFactory::CreateOWOFile(FString Filename, UObject* InParent, FName InName, EObjectFlags Flags)
{
    FString result;

    FFileHelper::LoadFileToString(result, *Filename);

    return NewObject<UImportedSensation>(InParent, UImportedSensation::StaticClass(), InName, Flags);
}