// Copyright (C) 2023 OWO Games - All Rights Reserved

#pragma once

#include "CoreMinimal.h"
#include "Engine/DataTable.h"
THIRD_PARTY_INCLUDES_START
#include "../OWOAPI/Domain/Sensation.h"
#include "../OWOAPI/Domain/Muscle.h"
#include "../OWOAPI/Domain/MusclesGroup.h"
#include "../OWOAPI/Domain/SensationsFactory.h"
#include "../OWOAPI/Domain/SensationsSequence.h"
#include "../OWOAPI/Domain/SensationsParser.h"
#include "../OWOAPI/Domain/MusclesParser.h"
THIRD_PARTY_INCLUDES_END
#include "OWOUnrealUtilities.generated.h"



#if WITH_EDITORONLY_DATA
class UAssetImportData;
#endif

USTRUCT(Blueprintable)
struct FOWOSensation : public FTableRowBase
{
	GENERATED_BODY()

public:
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "OWO", meta = (Tooltip = "The number of pulses per second the microsensation sends to you"))
	int Frequency;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "OWO", meta = (Tooltip = "The amount of time (in seconds) that the micro will last"))
	float DurationInSeconds;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "OWO", meta = (Tooltip = "The intensity percentage that users will feel based on their maximun intensity calibration"))
	int IntensityPercentage;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "OWO", meta = (Tooltip = "How long it takes the sensation to progressively go from 0 to the assigned intensity (in seconds)"))
	float RampUp;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "OWO", meta = (Tooltip = "How long it takes the sensation to progressively go from the assigned intensity to 0 (in seconds)"))
	float RampDown;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "OWO", meta = (Tooltip = "The amount of time (in seconds) that the micro will wait to send next sensation"))
	float ExitDelay;

};

UENUM(Blueprintable)
enum class EOWOConnectionState : uint8
{
	Disconnected = 0,
	Connecting = 1,
	Connected = 2,
};


UENUM(Blueprintable)
enum class EOWOMusclesEnum : uint8
{
	Pectoral_R = 0,
	Pectoral_L = 1,
	Abdominal_R = 2,
	Abdominal_L = 3,
	Arm_R = 4,
	Arm_L = 5,
	Dorsal_R = 6,
	Dorsal_L = 7,
	Lumbar_R = 8,
	Lumbar_L = 9
};

USTRUCT(Blueprintable)
struct FMuscleStruct
{
	GENERATED_BODY()

public:
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "OWO", meta = (Tooltip = "The muscle that will be affected by the sensation"))
		EOWOMusclesEnum Muscle;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "OWO", meta = (Tooltip = "The intensity percentage that will feel only in the selected muscle"))
		int IntensityPercentage = 100;

};

USTRUCT(Blueprintable)
struct FSensationData
{
	GENERATED_BODY()

public:
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Sensation")
		FOWOSensation SensationParameters;


	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Muscle")
		TArray<FMuscleStruct> MusclesParameters;

};

UCLASS(Blueprintable)
class OWOINTEGRATION_API UImportedAuth : public UObject 
{
	GENERATED_BODY()

public:
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Sensation")
		FString SensationsData;
#if WITH_EDITORONLY_DATA
	UPROPERTY(VisibleAnywhere, Instanced, Category = ImportSettings)
		class UAssetImportData* AssetImportData;
	void GetAssetRegistryTags(TArray<FAssetRegistryTag>& OutTags) const;
#endif

	void PostInitProperties();
	void PostLoad();

public:

	FString GetData();
};

UCLASS(Blueprintable)
class OWOINTEGRATION_API UImportedSensation : public UObject
{
	GENERATED_BODY()

public:
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Sensation")
		FString SensationsData;
#if WITH_EDITORONLY_DATA
	UPROPERTY(VisibleAnywhere, Instanced, Category = ImportSettings)
		class UAssetImportData* AssetImportData;
	void GetAssetRegistryTags(TArray<FAssetRegistryTag>& OutTags) const;
#endif

	void PostInitProperties();
	void PostLoad();

public:

	FString GetData();
};